@@ -17,9 +17,9 @@ from typing import List
 
 from eva.catalog.models.df_metadata import DataFrameMetadata
 from eva.expression.constant_value_expression import ConstantValueExpression
+from eva.parser.create_statement import ColumnDefinition
 from eva.parser.table_ref import TableRef
 from eva.expression.abstract_expression import AbstractExpression
-from eva.catalog.models.df_column import DataFrameColumn
 from eva.catalog.models.udf_io import UdfIO
 from pathlib import Path
 
@@ -42,24 +42,20 @@ class OperatorType(IntEnum):
     LOGICALORDERBY = auto()
     LOGICALLIMIT = auto()
     LOGICALSAMPLE = auto()
+    LOGICAL_CREATE_MATERIALIZED_VIEW = auto()
     LOGICALDELIMITER = auto()
 
 
 class Operator:
     """Base class for logital plan of operators
-
     Arguments:
         op_type: {OperatorType} -- {the opr type held by this node}
         children: {List} -- {the list of operator children for this node}
     """
 
-    def __init__(self, op_type: OperatorType, children: List = None):
+    def __init__(self, op_type: OperatorType, children=None):
         self._opr_type = op_type
-        self._children = children if children is not None else []
-
-    def append_child(self, child: 'Operator'):
-        if child:
-            self._children.append(child)
+        self._children = children or []
 
     @property
     def children(self):
@@ -73,6 +69,12 @@ class Operator:
     def opr_type(self):
         return self._opr_type
 
+    def append_child(self, child: 'Operator'):
+        self.children.append(child)
+
+    def clear_children(self):
+        self.children = []
+
     def __str__(self) -> str:
         return '%s[%s](%s)' % (
             type(self).__name__, hex(id(self)),
@@ -92,20 +94,50 @@ class Operator:
     def is_logical(self):
         return self._opr_type < OperatorType.LOGICALDELIMITER
 
+    def __hash__(self) -> int:
+        return hash((self.opr_type, tuple(self.children)))
+
+    def __copy__(self):
+        # deepcopy the children
+        cls = self.__class__
+        result = cls.__new__(cls)
+        for k, v in self.__dict__.items():
+            if k == '_children':
+                setattr(result, k, [])
+            else:
+                setattr(result, k, v)
+        return result
+
 
 class Dummy(Operator):
-    def __init__(self):
+    '''
+    Acts as a placeholder for matching any operator in optimizer.
+    It track the group_id of the matching operator.
+    '''
+
+    def __init__(self, group_id: int):
         super().__init__(OperatorType.DUMMY, None)
+        self.group_id = group_id
+
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), self.group_id))
 
 
 class LogicalGet(Operator):
-    def __init__(self, video: TableRef, dataset_metadata: DataFrameMetadata,
-                 children: List = None):
-        super().__init__(OperatorType.LOGICALGET, children)
+
+    def __init__(self,
+                 video: TableRef,
+                 dataset_metadata: DataFrameMetadata,
+                 alias: str,
+                 predicate: AbstractExpression = None,
+                 target_list: List[AbstractExpression] = None,
+                 children=None):
         self._video = video
         self._dataset_metadata = dataset_metadata
-        self._predicate = None
-        self._target_list = None
+        self._alias = alias
+        self._predicate = predicate
+        self._target_list = target_list or []
+        super().__init__(OperatorType.LOGICALGET, children)
 
     @property
     def video(self):
@@ -115,6 +147,10 @@ class LogicalGet(Operator):
     def dataset_metadata(self):
         return self._dataset_metadata
 
+    @property
+    def alias(self):
+        return self._alias
+
     @property
     def predicate(self):
         return self._predicate
@@ -138,35 +174,35 @@ class LogicalGet(Operator):
         return (is_subtree_equal
                 and self.video == other.video
                 and self.dataset_metadata == other.dataset_metadata
+                and self.alias == other.alias
                 and self.predicate == other.predicate
                 and self.target_list == other.target_list)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.alias,
+                     self.video,
+                     self.dataset_metadata,
+                     self.predicate,
+                     tuple(self.target_list)))
+
 
 class LogicalQueryDerivedGet(Operator):
-    def __init__(self, children: List = None):
+
+    def __init__(self,
+                 alias: str,
+                 predicate: AbstractExpression = None,
+                 target_list: List[AbstractExpression] = None,
+                 children: List = None):
         super().__init__(OperatorType.LOGICALQUERYDERIVEDGET,
                          children=children)
-        # `TODO` We need to store the alias information here
-        # We need construct the map using the target list of the
-        # subquery to validate the overall query
-        self.predicate = None
-        self.target_list = None
+        self._alias = alias
+        self.predicate = predicate
+        self.target_list = target_list or []
 
     @property
-    def predicate(self):
-        return self._predicate
-
-    @predicate.setter
-    def predicate(self, predicate):
-        self._predicate = predicate
-
-    @property
-    def target_list(self):
-        return self._target_list
-
-    @target_list.setter
-    def target_list(self, target_list):
-        self._target_list = target_list
+    def alias(self):
+        return self._alias
 
     def __eq__(self, other):
         is_subtree_equal = super().__eq__(other)
@@ -174,13 +210,20 @@ class LogicalQueryDerivedGet(Operator):
             return False
         return (is_subtree_equal
                 and self.predicate == other.predicate
-                and self.target_list == other.target_list)
+                and self.target_list == other.target_list
+                and self.alias == other.alias)
+
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.alias,
+                     self.predicate,
+                     tuple(self.target_list)))
 
 
 class LogicalFilter(Operator):
-    def __init__(self, predicate: AbstractExpression, children: List = None):
-        super().__init__(OperatorType.LOGICALFILTER, children)
+    def __init__(self, predicate: AbstractExpression, children=None):
         self._predicate = predicate
+        super().__init__(OperatorType.LOGICALFILTER, children)
 
     @property
     def predicate(self):
@@ -193,10 +236,13 @@ class LogicalFilter(Operator):
         return (is_subtree_equal
                 and self.predicate == other.predicate)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), self.predicate))
+
 
 class LogicalProject(Operator):
     def __init__(self, target_list: List[AbstractExpression],
-                 children: List = None):
+                 children=None):
         super().__init__(OperatorType.LOGICALPROJECT, children)
         self._target_list = target_list
 
@@ -211,6 +257,9 @@ class LogicalProject(Operator):
         return (is_subtree_equal
                 and self.target_list == other.target_list)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), tuple(self.target_list)))
+
 
 class LogicalOrderBy(Operator):
     def __init__(self, orderby_list: List,
@@ -229,9 +278,13 @@ class LogicalOrderBy(Operator):
         return (is_subtree_equal
                 and self.orderby_list == other.orderby_list)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), tuple(self.orderby_list)))
+
 
 class LogicalLimit(Operator):
-    def __init__(self, limit_count: ConstantValueExpression,
+    def __init__(self,
+                 limit_count: ConstantValueExpression,
                  children: List = None):
         super().__init__(OperatorType.LOGICALLIMIT, children)
         self._limit_count = limit_count
@@ -247,6 +300,9 @@ class LogicalLimit(Operator):
         return (is_subtree_equal
                 and self.limit_count == other.limit_count)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), self.limit_count))
+
 
 class LogicalSample(Operator):
     def __init__(self, sample_freq: ConstantValueExpression,
@@ -265,6 +321,9 @@ class LogicalSample(Operator):
         return (is_subtree_equal
                 and self.sample_freq == other.sample_freq)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), self.sample_freq))
+
 
 class LogicalUnion(Operator):
     def __init__(self, all: bool, children: List = None):
@@ -281,38 +340,33 @@ class LogicalUnion(Operator):
             return False
         return (is_subtree_equal and self.all == other.all)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(), self.all))
+
 
 class LogicalInsert(Operator):
     """[Logical Node for Insert operation]
 
     Arguments:
-        video {TableRef}:
-            [TableRef object copied from parsed statement]
-        video_catalog_id{int}:
-            [catalog id for the video table]
+        table_metainfo(DataFrameMetadata): table to intert data into
         column_list{List[AbstractExpression]}:
             [After binding annotated column_list]
         value_list{List[AbstractExpression]}:
             [value list to insert]
     """
 
-    def __init__(self, video: TableRef, video_catalog_id: int,
+    def __init__(self, table_metainfo: DataFrameMetadata,
                  column_list: List[AbstractExpression],
                  value_list: List[AbstractExpression],
                  children: List = None):
         super().__init__(OperatorType.LOGICALINSERT, children)
-        self._video = video
-        self._video_catalog_id = video_catalog_id
+        self._table_metainfo = table_metainfo
         self._column_list = column_list
         self._value_list = value_list
 
     @property
-    def video(self):
-        return self._video
-
-    @property
-    def video_catalog_id(self):
-        return self._video_catalog_id
+    def table_metainfo(self):
+        return self._table_metainfo
 
     @property
     def value_list(self):
@@ -327,25 +381,29 @@ class LogicalInsert(Operator):
         if not isinstance(other, LogicalInsert):
             return False
         return (is_subtree_equal
-                and self.video == other.video
-                and self.video_catalog_id == other.video_catalog_id
+                and self.table_metainfo == other.table_metainfo
                 and self.value_list == other.value_list
                 and self.column_list == other.column_list)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.table_metainfo,
+                     tuple(self.value_list),
+                     tuple(self.column_list)))
+
 
 class LogicalCreate(Operator):
     """Logical node for create table operations
 
     Arguments:
         video {TableRef}: [video table that is to be created]
-        column_list {List[DataFrameColumn]}:
-            [After binding annotated column_list]
+        column_list {List[ColumnDefinition]}:
         if_not_exists {bool}: [create table if exists]
 
     """
 
-    def __init__(self, video: TableRef, column_list: List[DataFrameColumn],
-                 if_not_exists: bool = False, children=None):
+    def __init__(self, video: TableRef, column_list: List[ColumnDefinition],
+                 if_not_exists: bool = False, children: List = None):
         super().__init__(OperatorType.LOGICALCREATE, children)
         self._video = video
         self._column_list = column_list
@@ -372,6 +430,12 @@ class LogicalCreate(Operator):
                 and self.column_list == other.column_list
                 and self.if_not_exists == other.if_not_exists)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.video,
+                     tuple(self.column_list),
+                     self.if_not_exists))
+
 
 class LogicalCreateUDF(Operator):
     """
@@ -401,7 +465,7 @@ class LogicalCreateUDF(Operator):
                  outputs: List[UdfIO],
                  impl_path: Path,
                  udf_type: str = None,
-                 children=None):
+                 children: List = None):
         super().__init__(OperatorType.LOGICALCREATEUDF, children)
         self._name = name
         self._if_not_exists = if_not_exists
@@ -446,6 +510,15 @@ class LogicalCreateUDF(Operator):
                 and self.udf_type == other.udf_type
                 and self.impl_path == other.impl_path)
 
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.name,
+                     self.if_not_exists,
+                     tuple(self.inputs),
+                     tuple(self.outputs),
+                     self.udf_type,
+                     self.impl_path))
+
 
 class LogicalLoadData(Operator):
     """Logical node for load data operation
@@ -456,10 +529,14 @@ class LogicalLoadData(Operator):
     """
 
     def __init__(self, table_metainfo: DataFrameMetadata,
-                 path: Path, children=None):
+                 path: Path,
+                 column_list: List[AbstractExpression] = None,
+                 file_options: dict = dict(), children: List = None):
         super().__init__(OperatorType.LOGICALLOADDATA, children=children)
         self._table_metainfo = table_metainfo
         self._path = path
+        self._column_list = column_list or []
+        self._file_options = file_options
 
     @property
     def table_metainfo(self):
@@ -469,9 +546,21 @@ class LogicalLoadData(Operator):
     def path(self):
         return self._path
 
+    @property
+    def column_list(self):
+        return self._column_list
+
+    @property
+    def file_options(self):
+        return self._file_options
+
     def __str__(self):
-        return 'LogicalLoadData(table: {}, path: {})'.format(
-            self.table_metainfo, self.path)
+        return "LogicalLoadData(table: {}, path: {}, \
+                column_list: {}, \
+                file_options: {})".format(self.table_metainfo,
+                                          self.path,
+                                          self.column_list,
+                                          self.file_options)
 
     def __eq__(self, other):
         is_subtree_equal = super().__eq__(other)
@@ -479,7 +568,16 @@ class LogicalLoadData(Operator):
             return False
         return (is_subtree_equal
                 and self.table_metainfo == other.table_metainfo
-                and self.path == other.path)
+                and self.path == other.path
+                and self.column_list == other.column_list
+                and self.file_options == other.file_options)
+
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.table_metainfo,
+                     self.path,
+                     tuple(self.column_list),
+                     frozenset(self.file_options.items())))
 
 
 class LogicalUpload(Operator):
@@ -515,3 +613,53 @@ class LogicalUpload(Operator):
         return (is_subtree_equal
                 and self.path == other.path
                 and self.video_blob == other.video_blob)
+
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.path,
+                     self.path,
+                     self.video_blob))
+
+
+class LogicalCreateMaterializedView(Operator):
+    """Logical node for create materiaziled view operations
+    Arguments:
+        view {TableRef}: [view table that is to be created]
+        col_list{List[ColumnDefinition]} -- column names in the view
+        if_not_exists {bool}: [whether to override if view exists]
+    """
+
+    def __init__(self, view: TableRef, col_list: List[ColumnDefinition],
+                 if_not_exists: bool = False, children=None):
+        super().__init__(OperatorType.LOGICAL_CREATE_MATERIALIZED_VIEW,
+                         children)
+        self._view = view
+        self._col_list = col_list
+        self._if_not_exists = if_not_exists
+
+    @property
+    def view(self):
+        return self._view
+
+    @property
+    def if_not_exists(self):
+        return self._if_not_exists
+
+    @property
+    def col_list(self):
+        return self._col_list
+
+    def __eq__(self, other):
+        is_subtree_equal = super().__eq__(other)
+        if not isinstance(other, LogicalCreateMaterializedView):
+            return False
+        return (is_subtree_equal
+                and self.view == other.view
+                and self.col_list == other.col_list
+                and self.if_not_exists == other.if_not_exists)
+
+    def __hash__(self) -> int:
+        return hash((super().__hash__(),
+                     self.view,
+                     tuple(self.col_list),
+                     self.if_not_exists))
