@@ -27,9 +27,10 @@ from eva.optimizer.operators import (
     LogicalCreate, LogicalInsert, LogicalLoadData, LogicalUpload,
     LogicalCreateUDF, LogicalProject, LogicalGet, LogicalFilter,
     LogicalUnion, LogicalOrderBy, LogicalLimit, LogicalQueryDerivedGet,
-    LogicalSample)
+    LogicalSample, LogicalCreateMaterializedView)
 from eva.planner.create_plan import CreatePlan
 from eva.planner.create_udf_plan import CreateUDFPlan
+from eva.planner.create_mat_view_plan import CreateMaterializedViewPlan
 from eva.planner.insert_plan import InsertPlan
 from eva.planner.load_data_plan import LoadDataPlan
 from eva.planner.upload_plan import UploadPlan
@@ -51,11 +52,11 @@ class RuleType(Flag):
 
     # REWRITE RULES(LOGICAL -> LOGICAL)
     EMBED_FILTER_INTO_GET = auto()
-    EMBED_PROJECT_INTO_GET = auto()
     EMBED_FILTER_INTO_DERIVED_GET = auto()
-    EMBED_PROJECT_INTO_DERIVED_GET = auto()
     PUSHDOWN_FILTER_THROUGH_SAMPLE = auto()
     PUSHDOWN_PROJECT_THROUGH_SAMPLE = auto()
+    EMBED_PROJECT_INTO_DERIVED_GET = auto()
+    EMBED_PROJECT_INTO_GET = auto()
 
     REWRITE_DELIMETER = auto()
 
@@ -68,11 +69,14 @@ class RuleType(Flag):
     LOGICAL_UPLOAD_TO_PHYSICAL = auto()
     LOGICAL_CREATE_TO_PHYSICAL = auto()
     LOGICAL_CREATE_UDF_TO_PHYSICAL = auto()
+    LOGICAL_MATERIALIZED_VIEW_TO_PHYSICAL = auto()
     LOGICAL_GET_TO_SEQSCAN = auto()
     LOGICAL_SAMPLE_TO_UNIFORMSAMPLE = auto()
     LOGICAL_DERIVED_GET_TO_PHYSICAL = auto()
     IMPLEMENTATION_DELIMETER = auto()
 
+    NUM_RULES = auto()
+
 
 class Promise(IntEnum):
     """
@@ -82,6 +86,7 @@ class Promise(IntEnum):
     """
     # IMPLEMENTATION RULES
     LOGICAL_UNION_TO_PHYSICAL = auto()
+    LOGICAL_MATERIALIZED_VIEW_TO_PHYSICAL = auto()
     LOGICAL_ORDERBY_TO_PHYSICAL = auto()
     LOGICAL_LIMIT_TO_PHYSICAL = auto()
     LOGICAL_INSERT_TO_PHYSICAL = auto()
@@ -128,34 +133,6 @@ class Rule(ABC):
     def pattern(self, pattern):
         self._pattern = pattern
 
-    @classmethod
-    def _compare_expr_with_pattern(cls,
-                                   grp_id,
-                                   context: OptimizerContext,
-                                   pattern) -> bool:
-        """check if the logical tree of the expression matches the
-            provided pattern
-        Args:
-            input_expr ([type]): expr to match
-            pattern: pattern to match with
-        Returns:
-            bool: If rule pattern matches, return true, else false
-        """
-        is_equal = True
-        grp = context.memo.groups[grp_id]
-        grp_expr = grp.get_logical_expr()
-        if grp_expr is None:
-            return False
-        if (grp_expr.opr.opr_type != pattern.opr_type or
-                (len(grp_expr.children) != len(pattern.children))):
-            return False
-        # recursively compare pattern and input_expr
-        for child_id, pattern_child in zip(grp_expr.children,
-                                           pattern.children):
-            is_equal &= cls._compare_expr_with_pattern(
-                child_id, context, pattern_child)
-        return is_equal
-
     def top_match(self, opr: Operator) -> bool:
         return opr.opr_type == self.pattern.opr_type
 
@@ -206,10 +183,13 @@ class EmbedFilterIntoGet(Rule):
 
     def apply(self, before: LogicalFilter, context: OptimizerContext):
         predicate = before.predicate
-        # logical_get = copy.deepcopy(before.children[0])
-        logical_get = before.children[0]
-        logical_get.predicate = predicate
-        return logical_get
+        lget = before.children[0]
+        new_get_opr = LogicalGet(lget.video, lget.dataset_metadata,
+                                 alias=lget.alias,
+                                 predicate=predicate,
+                                 target_list=lget.target_list,
+                                 children=lget.children)
+        return new_get_opr
 
 
 class EmbedProjectIntoGet(Rule):
@@ -226,11 +206,15 @@ class EmbedProjectIntoGet(Rule):
         return True
 
     def apply(self, before: LogicalProject, context: OptimizerContext):
-        select_list = before.target_list
-        # logical_get = copy.deepcopy(before.children[0])
-        logical_get = before.children[0]
-        logical_get.target_list = select_list
-        return logical_get
+        target_list = before.target_list
+        lget = before.children[0]
+        new_get_opr = LogicalGet(lget.video, lget.dataset_metadata,
+                                 alias=lget.alias,
+                                 predicate=lget.predicate,
+                                 target_list=target_list,
+                                 children=lget.children)
+
+        return new_get_opr
 
 # For nested queries
 
@@ -252,10 +236,12 @@ class EmbedFilterIntoDerivedGet(Rule):
 
     def apply(self, before: LogicalFilter, context: OptimizerContext):
         predicate = before.predicate
-        # logical_derived_get = copy.deepcopy(before.children[0])
-        logical_derived_get = before.children[0]
-        logical_derived_get.predicate = predicate
-        return logical_derived_get
+        ld_get = before.children[0]
+        new_opr = LogicalQueryDerivedGet(alias=ld_get.alias,
+                                         predicate=predicate,
+                                         target_list=ld_get.target_list,
+                                         children=ld_get.children)
+        return new_opr
 
 
 class EmbedProjectIntoDerivedGet(Rule):
@@ -274,11 +260,13 @@ class EmbedProjectIntoDerivedGet(Rule):
         return True
 
     def apply(self, before: LogicalProject, context: OptimizerContext):
-        select_list = before.target_list
-        # logical_derived_get = copy.deepcopy(before.children[0])
-        logical_derived_get = before.children[0]
-        logical_derived_get.target_list = select_list
-        return logical_derived_get
+        target_list = before.target_list
+        ld_get = before.children[0]
+        new_opr = LogicalQueryDerivedGet(alias=ld_get.alias,
+                                         predicate=ld_get.predicate,
+                                         target_list=target_list,
+                                         children=ld_get.children)
+        return new_opr
 
 
 class PushdownFilterThroughSample(Rule):
@@ -299,7 +287,10 @@ class PushdownFilterThroughSample(Rule):
     def apply(self, before: LogicalFilter, context: OptimizerContext):
         sample = before.children[0]
         logical_get = sample.children[0]
-        logical_get.predicate = before.predicate
+        new_filter = LogicalFilter(before.predicate)
+        new_filter.append_child(logical_get)
+        sample.clear_children()
+        sample.append_child(new_filter)
         return sample
 
 
@@ -321,7 +312,10 @@ class PushdownProjectThroughSample(Rule):
     def apply(self, before: LogicalProject, context: OptimizerContext):
         sample = before.children[0]
         logical_get = sample.children[0]
-        logical_get.target_list = before.target_list
+        new_project = LogicalProject(before.target_list)
+        new_project.append_child(logical_get)
+        sample.clear_children()
+        sample.append_child(new_project)
         return sample
 
 
@@ -386,7 +380,7 @@ class LogicalInsertToPhysical(Rule):
         return True
 
     def apply(self, before: LogicalInsert, context: OptimizerContext):
-        after = InsertPlan(before.video_catalog_id,
+        after = InsertPlan(before.table_metainfo,
                            before.column_list, before.value_list)
         return after
 
@@ -414,7 +408,10 @@ class LogicalLoadToPhysical(Rule):
         if config_batch_mem_size:
             batch_mem_size = config_batch_mem_size
         after = LoadDataPlan(before.table_metainfo,
-                             before.path, batch_mem_size)
+                             before.path,
+                             batch_mem_size,
+                             before.column_list,
+                             before.file_options)
         return after
 
 
@@ -456,7 +453,7 @@ class LogicalGetToSeqScan(Rule):
             "executor", "batch_mem_size")
         if config_batch_mem_size:
             batch_mem_size = config_batch_mem_size
-        after = SeqScanPlan(before.predicate, before.target_list)
+        after = SeqScanPlan(before.predicate, before.target_list, before.alias)
         after.append_child(StoragePlan(
             before.dataset_metadata, batch_mem_size=batch_mem_size))
         return after
@@ -476,6 +473,8 @@ class LogicalSampleToUniformSample(Rule):
 
     def apply(self, before: LogicalSample, context: OptimizerContext):
         after = SamplePlan(before.sample_freq)
+        for child in before.children:
+            after.append_child(child)
         return after
 
 
@@ -493,7 +492,8 @@ class LogicalDerivedGetToPhysical(Rule):
 
     def apply(self, before: LogicalQueryDerivedGet,
               context: OptimizerContext):
-        after = SeqScanPlan(before.predicate, before.target_list)
+        after = SeqScanPlan(before.predicate, before.target_list, before.alias)
+        after.append_child(before.children[0])
         return after
 
 
@@ -513,6 +513,8 @@ class LogicalUnionToPhysical(Rule):
 
     def apply(self, before: LogicalUnion, context: OptimizerContext):
         after = UnionPlan(before.all)
+        for child in before.children:
+            after.append_child(child)
         return after
 
 
@@ -530,6 +532,8 @@ class LogicalOrderByToPhysical(Rule):
 
     def apply(self, before: LogicalOrderBy, context: OptimizerContext):
         after = OrderByPlan(before.orderby_list)
+        for child in before.children:
+            after.append_child(child)
         return after
 
 
@@ -547,9 +551,33 @@ class LogicalLimitToPhysical(Rule):
 
     def apply(self, before: LogicalLimit, context: OptimizerContext):
         after = LimitPlan(before.limit_count)
+        for child in before.children:
+            after.append_child(child)
         return after
 
 
+class LogicalCreateMaterializedViewToPhysical(Rule):
+    def __init__(self):
+        pattern = Pattern(OperatorType.LOGICAL_CREATE_MATERIALIZED_VIEW)
+        pattern.append_child(Pattern(OperatorType.DUMMY))
+        super().__init__(RuleType.LOGICAL_MATERIALIZED_VIEW_TO_PHYSICAL,
+                         pattern)
+
+    def promise(self):
+        return Promise.LOGICAL_MATERIALIZED_VIEW_TO_PHYSICAL
+
+    def check(self, grp_id: int, context: OptimizerContext):
+        return True
+
+    def apply(self, before: LogicalCreateMaterializedView,
+              context: OptimizerContext):
+        after = CreateMaterializedViewPlan(before.view,
+                                           columns=before.col_list,
+                                           if_not_exists=before.if_not_exists)
+        for child in before.children:
+            after.append_child(child)
+        return after
+
 # IMPLEMENTATION RULES END
 ##############################################
 
@@ -565,12 +593,13 @@ class RulesManager:
         return cls._instance
 
     def __init__(self):
+
         self._rewrite_rules = [
             EmbedFilterIntoGet(),
-            EmbedProjectIntoGet(),
             EmbedFilterIntoDerivedGet(),
-            EmbedProjectIntoDerivedGet(),
             PushdownFilterThroughSample(),
+            EmbedProjectIntoGet(),
+            EmbedProjectIntoDerivedGet(),
             PushdownProjectThroughSample()
         ]
 
@@ -585,8 +614,10 @@ class RulesManager:
             LogicalDerivedGetToPhysical(),
             LogicalUnionToPhysical(),
             LogicalOrderByToPhysical(),
-            LogicalLimitToPhysical()
+            LogicalLimitToPhysical(),
+            LogicalCreateMaterializedViewToPhysical()
         ]
+        self._all_rules = self._rewrite_rules + self._implementation_rules
 
     @property
     def rewrite_rules(self):
@@ -595,3 +626,7 @@ class RulesManager:
     @property
     def implementation_rules(self):
         return self._implementation_rules
+
+    @property
+    def all_rules(self):
+        return self._all_rules
